searchData={"items":[{"type":"module","title":"Bunch","doc":"A bunch of general-purpose helper and convenience functions.","ref":"Bunch.html"},{"type":"macro","title":"Bunch.__using__/1","doc":"Imports a bunch of Bunch macros: `withl/1`, `withl/2`, `~>/2`, `~>>/2`, `quote_expr/1`, `quote_expr/2`","ref":"Bunch.html#__using__/1"},{"type":"function","title":"Bunch.error_if_nil/2","doc":"Returns an `:error` tuple if given value is `nil` and `:ok` tuple otherwise.","ref":"Bunch.html#error_if_nil/2"},{"type":"function","title":"Examples - Bunch.error_if_nil/2","doc":"iex> map = %{:answer => 42}\n    iex> Bunch.error_if_nil(map[:answer], :reason)\n    {:ok, 42}\n    iex> Bunch.error_if_nil(map[:invalid], :reason)\n    {:error, :reason}","ref":"Bunch.html#error_if_nil/2-examples"},{"type":"function","title":"Bunch.key/1","doc":"Extracts the key from a key-value tuple.","ref":"Bunch.html#key/1"},{"type":"function","title":"Bunch.listify/1","doc":"Embeds the argument in a one-element list if it is not a list itself. Otherwise\nworks as identity.\n\nWorks similarly to `List.wrap/1`, but treats `nil` as any non-list value,\ninstead of returning empty list in this case.","ref":"Bunch.html#listify/1"},{"type":"function","title":"Examples - Bunch.listify/1","doc":"iex> Bunch.listify(:a)\n    [:a]\n    iex> Bunch.listify([:a, :b, :c])\n    [:a, :b, :c]\n    iex> Bunch.listify(nil)\n    [nil]","ref":"Bunch.html#listify/1-examples"},{"type":"function","title":"Bunch.make_short_ref/0","doc":"Creates a short reference.","ref":"Bunch.html#make_short_ref/0"},{"type":"macro","title":"Bunch.quote_expr/2","doc":"Works like `quote/2`, but doesn't require a do/end block and options are passed\nas the last argument.\n\nUseful when quoting a single expression.","ref":"Bunch.html#quote_expr/2"},{"type":"macro","title":"Examples - Bunch.quote_expr/2","doc":"iex> use Bunch\n    iex> quote_expr(String.t())\n    quote do String.t() end\n    iex> quote_expr(unquote(x) + 2, unquote: false)\n    quote unquote: false do unquote(x) + 2 end","ref":"Bunch.html#quote_expr/2-examples"},{"type":"macro","title":"Nesting - Bunch.quote_expr/2","doc":"Nesting calls to `quote` disables unquoting in the inner call, while placing\n`quote_expr` in `quote` or another `quote_expr` does not:\n\n    iex> use Bunch\n    iex> quote do quote do unquote(:code) end end == quote do quote do :code end end\n    false\n    iex> quote do quote_expr(unquote(:code)) end == quote do quote_expr(:code) end\n    true","ref":"Bunch.html#quote_expr/2-nesting"},{"type":"function","title":"Bunch.stateful_try_with_status/1","doc":"Returns given stateful try value along with its status.","ref":"Bunch.html#stateful_try_with_status/1"},{"type":"function","title":"Bunch.then_if/3","doc":"Maps a value `x` with a `function` if the condition is true, acts as\nan identity function otherwise.","ref":"Bunch.html#then_if/3"},{"type":"function","title":"Examples - Bunch.then_if/3","doc":"iex> use Bunch\n    iex> then_if(1, false, & &1 + 1)\n    1\n    iex> then_if(1, true, & &1 + 1)\n    2\n    iex> arg = 1\n    iex> arg |> then_if(not is_list(arg), fn arg -> [arg] end) |> Enum.map(&(&1*2))\n    [2]","ref":"Bunch.html#then_if/3-examples"},{"type":"function","title":"Bunch.value/1","doc":"Extracts the value from a key-value tuple.","ref":"Bunch.html#value/1"},{"type":"macro","title":"Bunch.withl/1","doc":"Works like `withl/2`, but allows shorter syntax.","ref":"Bunch.html#withl/1"},{"type":"macro","title":"Examples - Bunch.withl/1","doc":"iex> use Bunch\n    iex> x = 1\n    iex> y = 2\n    iex> withl a: true   0,\n    ...>       b: false   rem(2) == 0,\n    ...>       do: {x, y},\n    ...>       else: (a: false -> {:error, :x}; b: true -> {:error, :y})\n    {:error, :y}\n\n\nFor more details and more verbose and readable syntax, check docs for `withl/2`.","ref":"Bunch.html#withl/1-examples"},{"type":"macro","title":"Bunch.withl/2","doc":"A labeled version of the `with/1` macro.\n\nThis macro works like `with/1`, but enforces user to mark corresponding `withl`\nand `else` clauses with the same label (atom). If a `withl` clause does not\nmatch, only the `else` clauses marked with the same label are matched against\nthe result.\n\n\n    iex> use Bunch\n    iex> names = %{1 => \"Harold\", 2 => \"Małgorzata\"}\n    iex> test = fn id ->\n    ...>   withl id: {int_id, _}           name: {:ok, name}       {:ok, \"The name is #{name}\"}\n    ...>   else\n    ...>     id: :error -> {:error, :invalid_id}\n    ...>     name: :error -> {:error, :name_not_found}\n    ...>   end\n    ...> end\n    iex> test.(\"1\")\n    {:ok, \"The name is Harold\"}\n    iex> test.(\"5\")\n    {:error, :name_not_found}\n    iex> test.(\"something\")\n    {:error, :invalid_id}\n\n\n`withl` clauses using no `  use Bunch\n    iex> names = %{1 => \"Harold\", 2 => \"Małgorzata\"}\n    iex> test = fn id ->\n    ...>   withl id: {int_id, _}           do: int_id = int_id + 1,\n    ...>         name: {:ok, name}       {:ok, \"The name is #{name}\"}\n    ...>   else\n    ...>     id: :error -> {:error, :invalid_id}\n    ...>     name: :error -> {:error, :name_not_found}\n    ...>   end\n    ...> end\n    iex> test.(\"0\")\n    {:ok, \"The name is Harold\"}\n\n\nAll the `withl` clauses that use `  use Bunch\n    iex> try do\n    ...>   Code.compile_quoted(quote do\n    ...>     withl a: a when a > 0             b: b when b > 0         {:ok, a + b}\n    ...>     else\n    ...>       a: _ -> :error\n    ...>     end\n    ...>   end)\n    ...> rescue\n    ...>   e -> e.description\n    ...> end\n    \"Label :b not present in withl else clauses\"","ref":"Bunch.html#withl/2"},{"type":"macro","title":"Variable scoping - Bunch.withl/2","doc":"Because the labels are resolved in the compile time, they make it possible to\naccess results of already succeeded matches from `else` clauses. This may help\nhandling errors, like below:\n\n\n    iex> use Bunch\n    iex> names = %{1 => \"Harold\", 2 => \"Małgorzata\"}\n    iex> test = fn id ->\n    ...>   withl id: {int_id, _}           do: int_id = int_id + 1,\n    ...>         name: {:ok, name}       {:ok, \"The name is #{name}\"}\n    ...>   else\n    ...>     id: :error -> {:error, :invalid_id}\n    ...>     name: :error -> {:ok, \"The name is Defaultius the #{int_id}th\"}\n    ...>   end\n    ...> end\n    iex> test.(\"0\")\n    {:ok, \"The name is Harold\"}\n    iex> test.(\"5\")\n    {:ok, \"The name is Defaultius the 6th\"}","ref":"Bunch.html#withl/2-variable-scoping"},{"type":"macro","title":"Duplicate labels - Bunch.withl/2","doc":"`withl` supports marking multiple `withl` clauses with the same label, however\nin that case all the `else` clauses marked with such label are simply put multiple\ntimes into the generated code. Note that this may lead to confusion, in particular\nwhen variables are rebound in `withl` clauses:\n\n\n    iex> use Bunch\n    iex> test = fn x ->\n    ...>   withl a: x when x > 1           do: x = x + 1,\n    ...>         a: x when x       :ok\n    ...>   else\n    ...>     a: x -> {:error, x}\n    ...>   end\n    ...> end\n    iex> test.(2)\n    :ok\n    iex> test.(1)\n    {:error, 1}\n    iex> test.(3)\n    {:error, 4}","ref":"Bunch.html#withl/2-duplicate-labels"},{"type":"macro","title":"Bunch.~>/2","doc":"Helper for writing pipeline-like syntax. Maps given value using match clauses\nor lambda-like syntax.","ref":"Bunch.html#~%3E/2"},{"type":"macro","title":"Examples - Bunch.~>/2","doc":"iex> use Bunch\n    iex> {:ok, 10} ~> ({:ok, x} -> x)\n    10\n    iex> 5 ~> &1 + 2\n    7\n\nLambda-like expressions are not converted to lambdas under the hood, but\nresult of `expr` is injected to `&1` at the compile time.\n\nUseful especially when dealing with a pipeline of operations (made up e.g.\nwith pipe (`|>`) operator) some of which are hard to express in such form:\n\n    iex> use Bunch\n    iex> [\"Joe\", \"truck\", \"jacket\"]\n    ...> |> Enum.map(&String.downcase/1)\n    ...> |> Enum.filter(& &1 |> String.starts_with?(\"j\"))\n    ...> ~> [\"Words:\" | &1]\n    ...> |> Enum.join(\"\\n\")\n    \"Words:\n    joe\n    jacket\"","ref":"Bunch.html#~%3E/2-examples"},{"type":"macro","title":"Bunch.~>>/2","doc":"Works similar to `~>/2`, but accepts only `->` clauses and appends default\nidentity clause at the end.","ref":"Bunch.html#~%3E%3E/2"},{"type":"macro","title":"Examples - Bunch.~>>/2","doc":"iex> use Bunch\n    iex> {:ok, 10} ~>> ({:ok, x} -> {:ok, x+1})\n    {:ok, 11}\n    iex> :error ~>> ({:ok, x} -> {:ok, x+1})\n    :error","ref":"Bunch.html#~%3E%3E/2-examples"},{"type":"module","title":"Bunch.Access","doc":"A bunch of functions for easier manipulation on terms of types implementing `Access`\nbehaviour.","ref":"Bunch.Access.html"},{"type":"macro","title":"Bunch.Access.__using__/1","doc":"Implements `Access` behaviour by delegating callbacks to `Map` module.\n\nAll the callbacks are overridable.","ref":"Bunch.Access.html#__using__/1"},{"type":"function","title":"Bunch.Access.delete_in/2","doc":"Works like `pop_in/2`, but discards returned value.","ref":"Bunch.Access.html#delete_in/2"},{"type":"function","title":"Examples - Bunch.Access.delete_in/2","doc":"iex> Bunch.Access.delete_in(%{a: %{b: 10}}, [:a, :b])\n    %{a: %{}}\n    iex> Bunch.Access.delete_in(%{a: 10}, :a)\n    %{}\n    iex> Bunch.Access.delete_in(10, [])\n    nil","ref":"Bunch.Access.html#delete_in/2-examples"},{"type":"function","title":"Bunch.Access.get_and_update_in/3","doc":"Works like `Kernel.get_and_update_in/3` with small differences.\n\nBehaviour differs in the following aspects:\n- empty lists of keys are allowed\n- single key does not have to be wrapped in a list","ref":"Bunch.Access.html#get_and_update_in/3"},{"type":"function","title":"Examples - Bunch.Access.get_and_update_in/3","doc":"iex> Bunch.Access.get_and_update_in(%{a: %{b: 10}}, [:a, :b], & {&1, &1 * 2})\n    {10, %{a: %{b: 20}}}\n    iex> Bunch.Access.get_and_update_in(%{a: 10}, :a, & {&1, &1 * 2})\n    {10, %{a: 20}}\n    iex> Bunch.Access.get_and_update_in(10, [], & {&1, &1 * 2})\n    {10, 20}","ref":"Bunch.Access.html#get_and_update_in/3-examples"},{"type":"function","title":"Bunch.Access.get_in/2","doc":"Works like `Kernel.get_in/2` with small differences.\n\nBehaviour differs in the following aspects:\n- empty lists of keys are allowed\n- single key does not have to be wrapped in a list","ref":"Bunch.Access.html#get_in/2"},{"type":"function","title":"Examples - Bunch.Access.get_in/2","doc":"iex> Bunch.Access.get_in(%{a: %{b: 10}}, [:a, :b])\n    10\n    iex> Bunch.Access.get_in(%{a: 10}, :a)\n    10\n    iex> Bunch.Access.get_in(%{a: %{b: 10}}, [])\n    %{a: %{b: 10}}","ref":"Bunch.Access.html#get_in/2-examples"},{"type":"function","title":"Bunch.Access.get_updated_in/3","doc":"Updates value at `keys` in a nested data structure and returns new value and updated structure.\n\nUses `get_and_update_in/3` under the hood.","ref":"Bunch.Access.html#get_updated_in/3"},{"type":"function","title":"Example - Bunch.Access.get_updated_in/3","doc":"iex> %{a: %{b: 10}} |> Bunch.Access.get_updated_in([:a, :b], & &1+1)\n    {11, %{a: %{b: 11}}}","ref":"Bunch.Access.html#get_updated_in/3-example"},{"type":"function","title":"Bunch.Access.pop_in/2","doc":"Works like `Kernel.pop_in/2` with small differences.\n\nBehaviour differs in the following aspects:\n- empty lists of keys are allowed\n- single key does not have to be wrapped in a list","ref":"Bunch.Access.html#pop_in/2"},{"type":"function","title":"Examples - Bunch.Access.pop_in/2","doc":"iex> Bunch.Access.pop_in(%{a: %{b: 10}}, [:a, :b])\n    {10, %{a: %{}}}\n    iex> Bunch.Access.pop_in(%{a: 10}, :a)\n    {10, %{}}\n    iex> Bunch.Access.pop_in(10, [])\n    {10, nil}","ref":"Bunch.Access.html#pop_in/2-examples"},{"type":"function","title":"Bunch.Access.put_in/3","doc":"Works like `Kernel.put_in/3` with small differences.\n\nBehaviour differs in the following aspects:\n- empty lists of keys are allowed\n- single key does not have to be wrapped in a list","ref":"Bunch.Access.html#put_in/3"},{"type":"function","title":"Examples - Bunch.Access.put_in/3","doc":"iex> Bunch.Access.put_in(%{a: %{b: 10}}, [:a, :b], 20)\n    %{a: %{b: 20}}\n    iex> Bunch.Access.put_in(%{a: 10}, :a, 20)\n    %{a: 20}\n    iex> Bunch.Access.put_in(%{a: %{b: 10}}, [], 20)\n    20","ref":"Bunch.Access.html#put_in/3-examples"},{"type":"function","title":"Bunch.Access.update_in/3","doc":"Works like `Kernel.update_in/3` with small differences.\n\nBehaviour differs in the following aspects:\n- empty lists of keys are allowed\n- single key does not have to be wrapped in a list","ref":"Bunch.Access.html#update_in/3"},{"type":"function","title":"Examples - Bunch.Access.update_in/3","doc":"iex> Bunch.Access.update_in(%{a: %{b: 10}}, [:a, :b], & &1 * 2)\n    %{a: %{b: 20}}\n    iex> Bunch.Access.update_in(%{a: 10}, :a, & &1 * 2)\n    %{a: 20}\n    iex> Bunch.Access.update_in(10, [], & &1 * 2)\n    20","ref":"Bunch.Access.html#update_in/3-examples"},{"type":"module","title":"Bunch.Binary","doc":"A bunch of helpers for manipulating binaries.","ref":"Bunch.Binary.html"},{"type":"function","title":"Bunch.Binary.chunk_every/2","doc":"Chunks given binary into parts of given size.\n\nRemaining part is cut off.","ref":"Bunch.Binary.html#chunk_every/2"},{"type":"function","title":"Examples - Bunch.Binary.chunk_every/2","doc":"iex>  > |> Bunch.Binary.chunk_every(2)\n    [ >,  >,  >]\n    iex>  > |> Bunch.Binary.chunk_every(2)\n    [ >,  >,  >]","ref":"Bunch.Binary.html#chunk_every/2-examples"},{"type":"function","title":"Bunch.Binary.chunk_every_rem/2","doc":"Chunks given binary into parts of given size.\n\nReturns list of chunks and remainder.","ref":"Bunch.Binary.html#chunk_every_rem/2"},{"type":"function","title":"Examples - Bunch.Binary.chunk_every_rem/2","doc":"iex>  > |> Bunch.Binary.chunk_every_rem(2)\n    {[ >,  >,  >],  >}\n    iex>  > |> Bunch.Binary.chunk_every_rem(2)\n    {[ >,  >,  >],  >}","ref":"Bunch.Binary.html#chunk_every_rem/2-examples"},{"type":"function","title":"Bunch.Binary.split_int_part/2","doc":"Returns a 2-tuple, where the first element is the result of `take_int_part(binary, i)`,\nand the second is the rest of `binary`.","ref":"Bunch.Binary.html#split_int_part/2"},{"type":"function","title":"Examples - Bunch.Binary.split_int_part/2","doc":"iex> import Bunch.Binary\n    iex> split_int_part( >, 3)\n    { >,  >}\n    iex> split_int_part( >, 4)\n    { >,  >}","ref":"Bunch.Binary.html#split_int_part/2-examples"},{"type":"function","title":"Bunch.Binary.take_int_part/2","doc":"Cuts off the smallest possible chunk from the end of `binary`, so that the\nsize of returned binary is an integer multiple of `i`.","ref":"Bunch.Binary.html#take_int_part/2"},{"type":"function","title":"Examples - Bunch.Binary.take_int_part/2","doc":"iex> import Bunch.Binary\n    iex> take_int_part( >, 3)\n     >\n    iex> take_int_part( >, 4)\n     >","ref":"Bunch.Binary.html#take_int_part/2-examples"},{"type":"module","title":"Bunch.Code","doc":"A bunch of helper functions for code compilation, code evaluation, and code loading.","ref":"Bunch.Code.html"},{"type":"macro","title":"Bunch.Code.peek_code/1","doc":"Takes a code block, expands macros inside and pretty prints it.","ref":"Bunch.Code.html#peek_code/1"},{"type":"macro","title":"Bunch.Code.stacktrace/0","doc":"Returns stacktrace as a string.\n\nThe stacktrace is formatted to the readable format.","ref":"Bunch.Code.html#stacktrace/0"},{"type":"module","title":"Bunch.Config","doc":"A bunch of helpers for parsing and validating configurations.","ref":"Bunch.Config.html"},{"type":"function","title":"Bunch.Config.parse/2","doc":"Parses `config` according to `fields_specs`.\n\n`fields_specs` consist of constraints on each field. Supported constraints are:\n  * validate - function determining if field's value is correct\n  * in - enumerable containing all valid values\n  * default - value returned if a field is not found in `config`\n  * require? - determines whether a field is required, defaults to `false` when `default` is set\n    and `true` when `default` is not set\n  * require_if - deprecated, pass function returning constraints instead\n\nInstead of a list of constraints, a function accepting fields parsed so far and returning\nthe constraints can be passed. If the function returns `nil`, field is considered non existent,\nas if it wasn't passed at all.","ref":"Bunch.Config.html#parse/2"},{"type":"function","title":"Examples - Bunch.Config.parse/2","doc":"iex> Bunch.Config.parse([a: 1, b: 2], a: [validate: & &1 > 0], b: [in: -2..2])\n    {:ok, %{a: 1, b: 2}}\n    iex> Bunch.Config.parse([a: 1, b: 4], a: [validate: & &1 > 0], b: [in: -2..2])\n    {:error, {:config_field, {:invalid_value, [key: :b, value: 4, reason: {:not_in, -2..2}]}}}\n    iex> Bunch.Config.parse(\n    ...> [a: 1, b: 2],\n    ...> a: [validate: & &1 > 0],\n    ...> b: [in: -2..2],\n    ...> c: [default: 5]\n    ...> )\n    {:ok, %{a: 1, b: 2, c: 5}}\n    iex> Bunch.Config.parse(\n    ...> [a: 1, b: 2],\n    ...> a: [validate: & &1 > 0],\n    ...> b: [in: -2..2],\n    ...> c: [require?: false]\n    ...> )\n    {:ok, %{a: 1, b: 2}}\n    iex> Bunch.Config.parse(\n    ...> [a: 1, b: 1],\n    ...> a: [validate: & &1 > 0],\n    ...> b: [in: -2..2],\n    ...> c: &(if &1.a == &1.b, do: [in: 0..1])\n    ...> )\n    {:error, {:config_field, {:key_not_found, :c}}}","ref":"Bunch.Config.html#parse/2-examples"},{"type":"module","title":"Bunch.Enum","doc":"A bunch of helper functions for manipulating enums.","ref":"Bunch.Enum.html"},{"type":"function","title":"Bunch.Enum.chunk_by_prev/3","doc":"Splits enumerable into chunks, and passes each chunk through `collector`.\n\nNew chunk is created each time `chunker` returns `false`. The `chunker` is passed\ncurrent and previous element of enumerable.","ref":"Bunch.Enum.html#chunk_by_prev/3"},{"type":"function","title":"Examples: - Bunch.Enum.chunk_by_prev/3","doc":"iex> Bunch.Enum.chunk_by_prev([1,2,5,5], fn x, y -> x - y   Bunch.Enum.chunk_by_prev([1,2,5,5], fn x, y -> x - y <= 2 end, &Enum.sum/1)\n    [3, 10]","ref":"Bunch.Enum.html#chunk_by_prev/3-examples"},{"type":"function","title":"Bunch.Enum.duplicates/2","doc":"Returns elements that occur at least `min_occurences` times in enumerable.\n\nResults are NOT ordered in any sensible way, neither is the order anyhow preserved,\nbut it is deterministic.","ref":"Bunch.Enum.html#duplicates/2"},{"type":"function","title":"Examples - Bunch.Enum.duplicates/2","doc":"iex> Bunch.Enum.duplicates([1,3,2,5,3,2,2])\n    [2, 3]\n    iex> Bunch.Enum.duplicates([1,3,2,5,3,2,2], 3)\n    [2]","ref":"Bunch.Enum.html#duplicates/2-examples"},{"type":"function","title":"Bunch.Enum.repeat/2","doc":"Generates a list by calling `i` times function `f`.\n\n\n    iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Enum.repeat(fn -> Agent.get_and_update(pid, &{&1, &1+1}) end, 4)\n    [0, 1, 2, 3]\n    iex> Bunch.Enum.repeat(fn -> :abc end, 0)\n    []","ref":"Bunch.Enum.html#repeat/2"},{"type":"function","title":"Bunch.Enum.repeated/2","doc":"Generates a list consisting of `i` values `v`.\n\n\n    iex> Bunch.Enum.repeated(:abc, 4)\n    [:abc, :abc, :abc, :abc]\n    iex> Bunch.Enum.repeated(:abc, 0)\n    []","ref":"Bunch.Enum.html#repeated/2"},{"type":"function","title":"Bunch.Enum.try_each/2","doc":"Works like `Enum.each/2`, but breaks on error.\n\nBehaves like `Enum.each/2` as long as given `fun` returns `:ok`.\nIf it happens to return `{:error, reason}`, traversal is stopped and the\nerror is returned.","ref":"Bunch.Enum.html#try_each/2"},{"type":"function","title":"Examples: - Bunch.Enum.try_each/2","doc":"iex> fun = fn 0 -> {:error, :zero}; x -> send(self(), 1/x); :ok end\n    iex> Bunch.Enum.try_each([1,2,3], fun)\n    :ok\n    iex> Bunch.Enum.try_each([1,0,3], fun)\n    {:error, :zero}","ref":"Bunch.Enum.html#try_each/2-examples"},{"type":"function","title":"Bunch.Enum.try_flat_map/2","doc":"Works like `Enum.flat_map/2`, but breaks on error.\n\nBehaves like `Enum.flat_map/2` as long as reducing function returns `{:ok, values}`.\nIf it happens to return `{:error, reason}`, reduction is stopped and the\nerror is returned.","ref":"Bunch.Enum.html#try_flat_map/2"},{"type":"function","title":"Examples: - Bunch.Enum.try_flat_map/2","doc":"iex> fun = fn 0 -> {:error, :zero}; x -> {:ok, [1/x, 2/x, 3/x]} end\n    iex> Bunch.Enum.try_flat_map([1,5,-2,8], fun)\n    {:ok, [1.0, 2.0, 3.0, 0.2, 0.4, 0.6, -0.5, -1.0, -1.5, 0.125, 0.25, 0.375]}\n    iex> Bunch.Enum.try_flat_map([1,5,0,8], fun)\n    {:error, :zero}","ref":"Bunch.Enum.html#try_flat_map/2-examples"},{"type":"function","title":"Bunch.Enum.try_flat_map_reduce/3","doc":"Works like `Enum.each/2`, but breaks on error.\n\nBehaves like `Enum.flat_map_reduce/3` as long as given `fun` returns\n`{{:ok, value}, new_acc}`. If it happens to return `{{:error, reason}, new_acc}`,\nreduction is stopped and the error is returned.","ref":"Bunch.Enum.html#try_flat_map_reduce/3"},{"type":"function","title":"Examples: - Bunch.Enum.try_flat_map_reduce/3","doc":"iex> fun = fn\n    ...> x, acc when acc >= 0 -> {{:ok, [x+1, x+2, x+3]}, x + acc}\n    ...> _, acc -> {{:error, :negative_prefix_sum}, acc}\n    ...> end\n    iex> Bunch.Enum.try_flat_map_reduce([1,5,-2,8], 0, fun)\n    {{:ok, [2,3,4,6,7,8,-1,0,1,9,10,11]}, 12}\n    iex> Bunch.Enum.try_flat_map_reduce([1,5,-7,8], 0, fun)\n    {{:error, :negative_prefix_sum}, -1}","ref":"Bunch.Enum.html#try_flat_map_reduce/3-examples"},{"type":"function","title":"Bunch.Enum.try_map/2","doc":"Works like `Enum.map/2`, but breaks on error.\n\nBehaves like `Enum.map/2` as long as given `fun` returns `{:ok, value}`.\nIf it happens to return `{:error, reason}`, reduction is stopped and the\nerror is returned.","ref":"Bunch.Enum.html#try_map/2"},{"type":"function","title":"Examples: - Bunch.Enum.try_map/2","doc":"iex> fun = fn 0 -> {:error, :zero}; x -> {:ok, 1/x} end\n    iex> Bunch.Enum.try_map([1,5,-2,8], fun)\n    {:ok, [1.0, 0.2, -0.5, 0.125]}\n    iex> Bunch.Enum.try_map([1,5,0,8], fun)\n    {:error, :zero}","ref":"Bunch.Enum.html#try_map/2-examples"},{"type":"function","title":"Bunch.Enum.try_map_reduce/3","doc":"Works like `Enum.map_reduce/3`, but breaks on error.\n\nBehaves like `Enum.map_reduce/3` as long as given `fun` returns\n`{{:ok, value}, new_acc}`. If it happens to return `{{:error, reason}, new_acc}`,\nreduction is stopped and the error is returned.","ref":"Bunch.Enum.html#try_map_reduce/3"},{"type":"function","title":"Examples: - Bunch.Enum.try_map_reduce/3","doc":"iex> fun = fn\n    ...> x, acc when acc >= 0 -> {{:ok, x+1}, x + acc}\n    ...> _, acc -> {{:error, :negative_prefix_sum}, acc}\n    ...> end\n    iex> Bunch.Enum.try_map_reduce([1,5,-2,8], 0, fun)\n    {{:ok, [2,6,-1,9]}, 12}\n    iex> Bunch.Enum.try_map_reduce([1,5,-7,8], 0, fun)\n    {{:error, :negative_prefix_sum}, -1}","ref":"Bunch.Enum.html#try_map_reduce/3-examples"},{"type":"function","title":"Bunch.Enum.try_reduce/3","doc":"Works like `Enum.reduce/3`, but breaks on error.\n\nBehaves like `Enum.reduce/3` as long as given `fun` returns `{:ok, new_acc}`.\nIf it happens to return `{{:error, reason}, new_acc}`, reduction is stopped and\nthe error is returned.","ref":"Bunch.Enum.html#try_reduce/3"},{"type":"function","title":"Examples: - Bunch.Enum.try_reduce/3","doc":"iex> fun = fn\n    ...> x, acc when acc >= 0 -> {:ok,  x + acc}\n    ...> _, acc -> {{:error, :negative_prefix_sum}, acc}\n    ...> end\n    iex> Bunch.Enum.try_reduce([1,5,-2,8], 0, fun)\n    {:ok, 12}\n    iex> Bunch.Enum.try_reduce([1,5,-7,8], 0, fun)\n    {{:error, :negative_prefix_sum}, -1}","ref":"Bunch.Enum.html#try_reduce/3-examples"},{"type":"function","title":"Bunch.Enum.try_reduce_while/3","doc":"Works like `Enum.reduce_while/3`, but breaks on error.\n\nBehaves like `Enum.reduce_while/3` as long as given `fun` returns\n`{{:ok, :cont | :halt}, new_acc}`. If it happens to return\n`{{:error, reason}, new_acc}`, reduction is stopped and the error is returned.","ref":"Bunch.Enum.html#try_reduce_while/3"},{"type":"function","title":"Examples: - Bunch.Enum.try_reduce_while/3","doc":"iex> fun = fn\n    ...> 0, acc -> {{:ok, :halt}, acc}\n    ...> x, acc when acc >= 0 -> {{:ok, :cont}, x + acc}\n    ...> _, acc -> {{:error, :negative_prefix_sum}, acc}\n    ...> end\n    iex> Bunch.Enum.try_reduce_while([1,5,-2,8], 0, fun)\n    {:ok, 12}\n    iex> Bunch.Enum.try_reduce_while([1,5,0,8], 0, fun)\n    {:ok, 6}\n    iex> Bunch.Enum.try_reduce_while([1,5,-7,8], 0, fun)\n    {{:error, :negative_prefix_sum}, -1}","ref":"Bunch.Enum.html#try_reduce_while/3-examples"},{"type":"function","title":"Bunch.Enum.unzip/1","doc":"Implementation of `Enum.unzip/1` for more-than-two-element tuples.\n\nSize of returned tuple is equal to size of the shortest tuple in `tuples`.","ref":"Bunch.Enum.html#unzip/1"},{"type":"function","title":"Examples: - Bunch.Enum.unzip/1","doc":"iex> Bunch.Enum.unzip([{1,2,3}, {4,5,6}, {7,8,9}, {10,11,12}])\n    {[1, 4, 7, 10], [2, 5, 8, 11], [3, 6, 9, 12]}\n    iex> Bunch.Enum.unzip([{1,2,3}, {4,5}, {6,7,8,9}, {10,11,12}])\n    {[1, 4, 6, 10], [2, 5, 7, 11]}","ref":"Bunch.Enum.html#unzip/1-examples"},{"type":"function","title":"Bunch.Enum.zip_longest/1","doc":"Works the same way as `Enum.zip/1`, but does not cut off remaining values.","ref":"Bunch.Enum.html#zip_longest/1"},{"type":"function","title":"Examples: - Bunch.Enum.zip_longest/1","doc":"iex> Bunch.Enum.zip_longest([[1, 2] ,[3 ,4, 5]])\n    [[1, 3], [2, 4], [5]]\n\nIt also returns list of lists, as opposed to tuples.","ref":"Bunch.Enum.html#zip_longest/1-examples"},{"type":"module","title":"Bunch.KVEnum","doc":"A bunch of helper functions for manipulating key-value enums (including keyword\nenums).\n\nKey-value enums are represented as enums of 2-element tuples, where the first\nelement of each tuple is a key, and the second is a value.","ref":"Bunch.KVEnum.html"},{"type":"function","title":"Bunch.KVEnum.any_key?/2","doc":"Returns `true` if `f` returns truthy value for any key from `enum`, otherwise `false`.","ref":"Bunch.KVEnum.html#any_key?/2"},{"type":"function","title":"Example - Bunch.KVEnum.any_key?/2","doc":"iex> Bunch.KVEnum.any_key?([a: 1, b: 2, a: 3], & &1 == :b)\n    true\n    iex> Bunch.KVEnum.any_key?([a: 1, b: 3, a: 5], & &1 == :c)\n    false","ref":"Bunch.KVEnum.html#any_key?/2-example"},{"type":"function","title":"Bunch.KVEnum.any_value?/2","doc":"Returns `true` if `f` returns truthy value for any value from `enum`, otherwise `false`.","ref":"Bunch.KVEnum.html#any_value?/2"},{"type":"function","title":"Example - Bunch.KVEnum.any_value?/2","doc":"iex> Bunch.KVEnum.any_value?([a: 1, b: 2, a: 3], & &1 |> rem(2) == 0)\n    true\n    iex> Bunch.KVEnum.any_value?([a: 1, b: 3, a: 5], & &1 |> rem(2) == 0)\n    false","ref":"Bunch.KVEnum.html#any_value?/2-example"},{"type":"function","title":"Bunch.KVEnum.each_key/2","doc":"Executes `f` for each key in `enum`.","ref":"Bunch.KVEnum.html#each_key/2"},{"type":"function","title":"Example - Bunch.KVEnum.each_key/2","doc":"iex> Bunch.KVEnum.each_key([a: 1, b: 2, a: 3], & send(self(), &1))\n    iex> [:a, :b, :a] |> Enum.each(&receive do ^&1 -> :ok end)\n    :ok","ref":"Bunch.KVEnum.html#each_key/2-example"},{"type":"function","title":"Bunch.KVEnum.each_value/2","doc":"Executes `f` for each value in `enum`.","ref":"Bunch.KVEnum.html#each_value/2"},{"type":"function","title":"Example - Bunch.KVEnum.each_value/2","doc":"iex> Bunch.KVEnum.each_value([a: 1, b: 2, a: 3], & send(self(), &1))\n    iex> 1..3 |> Enum.each(&receive do ^&1 -> :ok end)\n    :ok","ref":"Bunch.KVEnum.html#each_value/2-example"},{"type":"function","title":"Bunch.KVEnum.filter_by_keys/2","doc":"Filters elements of `enum` by keys using function `f`.","ref":"Bunch.KVEnum.html#filter_by_keys/2"},{"type":"function","title":"Example - Bunch.KVEnum.filter_by_keys/2","doc":"iex> Bunch.KVEnum.filter_by_keys([a: 1, b: 2, a: 3], & &1 == :a)\n    [a: 1, a: 3]","ref":"Bunch.KVEnum.html#filter_by_keys/2-example"},{"type":"function","title":"Bunch.KVEnum.filter_by_values/2","doc":"Filters elements of `enum` by values using function `f`.","ref":"Bunch.KVEnum.html#filter_by_values/2"},{"type":"function","title":"Example - Bunch.KVEnum.filter_by_values/2","doc":"iex> Bunch.KVEnum.filter_by_values([a: 1, b: 2, a: 3], & &1 |> rem(2) == 0)\n    [b: 2]","ref":"Bunch.KVEnum.html#filter_by_values/2-example"},{"type":"function","title":"Bunch.KVEnum.keys/1","doc":"Returns all keys from the `enum`.\n\nDuplicated keys appear duplicated in the final enum of keys.","ref":"Bunch.KVEnum.html#keys/1"},{"type":"function","title":"Examples - Bunch.KVEnum.keys/1","doc":"iex> Bunch.KVEnum.keys(a: 1, b: 2)\n    [:a, :b]\n    iex> Bunch.KVEnum.keys(a: 1, b: 2, a: 3)\n    [:a, :b, :a]","ref":"Bunch.KVEnum.html#keys/1-examples"},{"type":"function","title":"Bunch.KVEnum.map_keys/2","doc":"Maps keys of `enum` using function `f`.","ref":"Bunch.KVEnum.html#map_keys/2"},{"type":"function","title":"Example - Bunch.KVEnum.map_keys/2","doc":"iex> Bunch.KVEnum.map_keys([{1, :a}, {2, :b}], & &1+1)\n    [{2, :a}, {3, :b}]","ref":"Bunch.KVEnum.html#map_keys/2-example"},{"type":"function","title":"Bunch.KVEnum.map_values/2","doc":"Maps values of `enum` using function `f`.","ref":"Bunch.KVEnum.html#map_values/2"},{"type":"function","title":"Example - Bunch.KVEnum.map_values/2","doc":"iex> Bunch.KVEnum.map_values([a: 1, b: 2], & &1+1)\n    [a: 2, b: 3]","ref":"Bunch.KVEnum.html#map_values/2-example"},{"type":"function","title":"Bunch.KVEnum.values/1","doc":"Returns all values from the `enum`.\n\nValues from duplicated keys will be kept in the final enum of values.","ref":"Bunch.KVEnum.html#values/1"},{"type":"function","title":"Examples - Bunch.KVEnum.values/1","doc":"iex> Bunch.KVEnum.values(a: 1, b: 2)\n    [1, 2]\n    iex> Bunch.KVEnum.values(a: 1, b: 2, a: 3)\n    [1, 2, 3]","ref":"Bunch.KVEnum.html#values/1-examples"},{"type":"type","title":"Bunch.KVEnum.t/2","doc":"","ref":"Bunch.KVEnum.html#t:t/2"},{"type":"module","title":"Bunch.KVList","doc":"A bunch of helper functions for manipulating key-value lists (including keyword\nlists).\n\nKey-value lists are represented as lists of 2-element tuples, where the first\nelement of each tuple is a key, and the second is a value.","ref":"Bunch.KVList.html"},{"type":"function","title":"Bunch.KVList.any_key?/2","doc":"Returns `true` if `f` returns truthy value for any key from `list`, otherwise `false`.","ref":"Bunch.KVList.html#any_key?/2"},{"type":"function","title":"Example - Bunch.KVList.any_key?/2","doc":"iex> Bunch.KVList.any_key?([a: 1, b: 2, a: 3], & &1 == :b)\n    true\n    iex> Bunch.KVList.any_key?([a: 1, b: 3, a: 5], & &1 == :c)\n    false","ref":"Bunch.KVList.html#any_key?/2-example"},{"type":"function","title":"Bunch.KVList.any_value?/2","doc":"Returns `true` if `f` returns truthy value for any value from `list`, otherwise `false`.","ref":"Bunch.KVList.html#any_value?/2"},{"type":"function","title":"Example - Bunch.KVList.any_value?/2","doc":"iex> Bunch.KVList.any_value?([a: 1, b: 2, a: 3], & &1 |> rem(2) == 0)\n    true\n    iex> Bunch.KVList.any_value?([a: 1, b: 3, a: 5], & &1 |> rem(2) == 0)\n    false","ref":"Bunch.KVList.html#any_value?/2-example"},{"type":"function","title":"Bunch.KVList.each_key/2","doc":"Executes `f` for each key in `list`.","ref":"Bunch.KVList.html#each_key/2"},{"type":"function","title":"Example - Bunch.KVList.each_key/2","doc":"iex> Bunch.KVList.each_key([a: 1, b: 2, a: 3], & send(self(), &1))\n    iex> [:a, :b, :a] |> Enum.each(&receive do ^&1 -> :ok end)\n    :ok","ref":"Bunch.KVList.html#each_key/2-example"},{"type":"function","title":"Bunch.KVList.each_value/2","doc":"Executes `f` for each value in `list`.","ref":"Bunch.KVList.html#each_value/2"},{"type":"function","title":"Example - Bunch.KVList.each_value/2","doc":"iex> Bunch.KVList.each_value([a: 1, b: 2, a: 3], & send(self(), &1))\n    iex> 1..3 |> Enum.each(&receive do ^&1 -> :ok end)\n    :ok","ref":"Bunch.KVList.html#each_value/2-example"},{"type":"function","title":"Bunch.KVList.filter_by_keys/2","doc":"Filters elements of `list` by keys using function `f`.","ref":"Bunch.KVList.html#filter_by_keys/2"},{"type":"function","title":"Example - Bunch.KVList.filter_by_keys/2","doc":"iex> Bunch.KVList.filter_by_keys([a: 1, b: 2, a: 3], & &1 == :a)\n    [a: 1, a: 3]","ref":"Bunch.KVList.html#filter_by_keys/2-example"},{"type":"function","title":"Bunch.KVList.filter_by_values/2","doc":"Filters elements of `list` by values using function `f`.","ref":"Bunch.KVList.html#filter_by_values/2"},{"type":"function","title":"Example - Bunch.KVList.filter_by_values/2","doc":"iex> Bunch.KVList.filter_by_values([a: 1, b: 2, a: 3], & &1 |> rem(2) == 0)\n    [b: 2]","ref":"Bunch.KVList.html#filter_by_values/2-example"},{"type":"function","title":"Bunch.KVList.map_keys/2","doc":"Maps keys of `list` using function `f`.","ref":"Bunch.KVList.html#map_keys/2"},{"type":"function","title":"Example - Bunch.KVList.map_keys/2","doc":"iex> Bunch.KVList.map_keys([{1, :a}, {2, :b}], & &1+1)\n    [{2, :a}, {3, :b}]","ref":"Bunch.KVList.html#map_keys/2-example"},{"type":"function","title":"Bunch.KVList.map_values/2","doc":"Maps values of `list` using function `f`.","ref":"Bunch.KVList.html#map_values/2"},{"type":"function","title":"Example - Bunch.KVList.map_values/2","doc":"iex> Bunch.KVList.map_values([a: 1, b: 2], & &1+1)\n    [a: 2, b: 3]","ref":"Bunch.KVList.html#map_values/2-example"},{"type":"type","title":"Bunch.KVList.t/2","doc":"","ref":"Bunch.KVList.html#t:t/2"},{"type":"module","title":"Bunch.List","doc":"A bunch of helper functions for list manipulation.","ref":"Bunch.List.html"},{"type":"function","title":"Bunch.List.try_unfoldr/2","doc":"The version of `unfoldr/2` that res_accepts `:ok` and `:error` return tuples.\n\nBehaves as `unfoldr/2` as long as `:ok` tuples are returned. Upon `:error`\nthe processing is stopped and error is returned.","ref":"Bunch.List.html#try_unfoldr/2"},{"type":"function","title":"Examples - Bunch.List.try_unfoldr/2","doc":"iex> f = fn\n    iex>    > ->\n    iex>     sum = a + b\n    iex>     if rem(sum, 2) == 1, do: {:ok, {:cont, sum, rest}}, else: {:error, :even_sum}\n    iex>   acc -> {:ok, {:halt, acc}}\n    iex> end\n    iex> Bunch.List.try_unfoldr( >, f)\n    {:ok, {[3, 7],  >}}\n    iex> Bunch.List.try_unfoldr( >, f)\n    {:error, :even_sum}","ref":"Bunch.List.html#try_unfoldr/2-examples"},{"type":"function","title":"Bunch.List.unfoldr/2","doc":"Generates a list using generator function and provided initial accumulator.\n\nSuccessive list elements are generated by calling `f` with the previous accumulator.\nThe enumeration finishes when it returns `:halt` or `{:halt, accumulator}`.","ref":"Bunch.List.html#unfoldr/2"},{"type":"function","title":"Examples - Bunch.List.unfoldr/2","doc":"iex> Bunch.List.unfoldr(10, fn 0 -> :halt; n -> {:cont, n-1} end)\n    Enum.to_list(9..0)\n    iex> f = fn\n    ...>    > -> {:cont, content, rest}\n    ...>   binary -> {:halt, binary}\n    ...> end\n    iex> Bunch.List.unfoldr( >, f)\n    {~w(ab cde fghi),  >}\n    iex> Bunch.List.unfoldr( >, f)\n    {~w(ab cde),  >}","ref":"Bunch.List.html#unfoldr/2-examples"},{"type":"module","title":"Bunch.Macro","doc":"A bunch of helpers for implementing macros.","ref":"Bunch.Macro.html"},{"type":"function","title":"Bunch.Macro.expand_deep/2","doc":"Receives an AST and traverses it expanding all the nodes.\n\nThis function uses `Macro.expand/2` under the hood. Check\nit out for more information and examples.","ref":"Bunch.Macro.html#expand_deep/2"},{"type":"function","title":"Bunch.Macro.inject_call/2","doc":"Imitates `import` functionality by finding and replacing bare function\ncalls (like `foo()`) in AST with fully-qualified call (like `Some.Module.foo()`)\n\nReceives AST fragment as first parameter and\na pair {Some.Module, :foo} as second","ref":"Bunch.Macro.html#inject_call/2"},{"type":"function","title":"Bunch.Macro.inject_calls/2","doc":"Imitates `import` functionality by finding and replacing bare function\ncalls (like `foo()`) in AST with fully-qualified call (like `Some.Module.foo()`)\n\nReceives AST fragment as first parameter and\nlist of pairs {Some.Module, :foo} as second","ref":"Bunch.Macro.html#inject_calls/2"},{"type":"function","title":"Bunch.Macro.prewalk_while/2","doc":"Works like `Macro.prewalk/2`, but allows to skip particular nodes.","ref":"Bunch.Macro.html#prewalk_while/2"},{"type":"function","title":"Example - Bunch.Macro.prewalk_while/2","doc":"iex> code = quote do fun(1, 2, opts: [key: :val]) end\n    iex> code |> Bunch.Macro.prewalk_while(fn node ->\n    ...>   if Keyword.keyword?(node) do\n    ...>     {:skip, node ++ [default: 1]}\n    ...>   else\n    ...>     {:enter, node}\n    ...>   end\n    ...> end)\n    quote do fun(1, 2, opts: [key: :val], default: 1) end","ref":"Bunch.Macro.html#prewalk_while/2-example"},{"type":"function","title":"Bunch.Macro.prewalk_while/3","doc":"Works like `Macro.prewalk/3`, but allows to skip particular nodes using an accumulator.","ref":"Bunch.Macro.html#prewalk_while/3"},{"type":"function","title":"Example - Bunch.Macro.prewalk_while/3","doc":"iex> code = quote do fun(1, 2, opts: [key: :val]) end\n    iex> code |> Bunch.Macro.prewalk_while(0, fn node, acc ->\n    ...>   if Keyword.keyword?(node) do\n    ...>     {:skip, node ++ [default: 1], acc + 1}\n    ...>   else\n    ...>     {:enter, node, acc}\n    ...>   end\n    ...> end)\n    {quote do fun(1, 2, opts: [key: :val], default: 1) end, 1}","ref":"Bunch.Macro.html#prewalk_while/3-example"},{"type":"module","title":"Bunch.Map","doc":"A bunch of helper functions for manipulating maps.","ref":"Bunch.Map.html"},{"type":"function","title":"Bunch.Map.get_updated/3","doc":"Updates value at `key` in `map` and returns new value and updated map.\n\nUses `Map.get_and_update/3` under the hood.","ref":"Bunch.Map.html#get_updated/3"},{"type":"function","title":"Example - Bunch.Map.get_updated/3","doc":"iex> %{a: 1} |> Bunch.Map.get_updated(:a, & &1+1)\n    {2, %{a: 2}}","ref":"Bunch.Map.html#get_updated/3-example"},{"type":"function","title":"Bunch.Map.get_updated!/3","doc":"Works like `get_updated/3`, but requires `map` to contain `key`.\n\nUses `Map.get_and_update!/3` under the hood.","ref":"Bunch.Map.html#get_updated!/3"},{"type":"function","title":"Example - Bunch.Map.get_updated!/3","doc":"iex> %{a: 1} |> Bunch.Map.get_updated!(:a, & &1+1)\n    {2, %{a: 2}}","ref":"Bunch.Map.html#get_updated!/3-example"},{"type":"function","title":"Bunch.Map.map_keys/2","doc":"Maps keys of `map` using function `f`.","ref":"Bunch.Map.html#map_keys/2"},{"type":"function","title":"Example - Bunch.Map.map_keys/2","doc":"iex> Bunch.Map.map_keys(%{1 => :a, 2 => :b}, & &1+1)\n    %{2 => :a, 3 => :b}","ref":"Bunch.Map.html#map_keys/2-example"},{"type":"function","title":"Bunch.Map.map_values/2","doc":"Maps values of `map` using function `f`.","ref":"Bunch.Map.html#map_values/2"},{"type":"function","title":"Example - Bunch.Map.map_values/2","doc":"iex> Bunch.Map.map_values(%{a: 1, b: 2}, & &1+1)\n    %{a: 2, b: 3}","ref":"Bunch.Map.html#map_values/2-example"},{"type":"function","title":"Bunch.Map.move/4","doc":"Moves value stored at `old_key` to `new_key`.\n\nIf `old_key` is not present in `map`, `default_value` is stored at `new_key`.\nIf `new_key` is present in `map`, it's value is overwritten.","ref":"Bunch.Map.html#move/4"},{"type":"function","title":"Examples - Bunch.Map.move/4","doc":"iex> Bunch.Map.move(%{a: 1, b: 2}, :a, :c, 3)\n    %{b: 2, c: 1}\n    iex> Bunch.Map.move(%{a: 1, b: 2}, :a, :b, 3)\n    %{b: 1}\n    iex> Bunch.Map.move(%{a: 1, b: 2}, :c, :b, 3)\n    %{a: 1, b: 3}","ref":"Bunch.Map.html#move/4-examples"},{"type":"function","title":"Bunch.Map.move!/3","doc":"Works like `move/3`, but fails if either `old_key` is absent or `new_key` is present\nin `map`.","ref":"Bunch.Map.html#move!/3"},{"type":"function","title":"Example - Bunch.Map.move!/3","doc":"iex> Bunch.Map.move!(%{a: 1, b: 2}, :a, :c)\n    %{b: 2, c: 1}","ref":"Bunch.Map.html#move!/3-example"},{"type":"module","title":"Bunch.Markdown","doc":"A bunch of helpers for generating Markdown text","ref":"Bunch.Markdown.html"},{"type":"function","title":"Bunch.Markdown.hard_indent/2","doc":"Indents the whole block of text by specified number of hard spaces (`&nbsp;`).","ref":"Bunch.Markdown.html#hard_indent/2"},{"type":"function","title":"Examples - Bunch.Markdown.hard_indent/2","doc":"iex>Bunch.Markdown.hard_indent(\"text\")\n    \"&nbsp;&nbsp;text\"\n\n    iex>text = \"\"\"\n    ...>First line\n    ...>Second line\n    ...>Third line\n    ...>\"\"\"\n    iex>Bunch.Markdown.hard_indent(text)\n    \"\"\"\n    &nbsp;&nbsp;First line\n    &nbsp;&nbsp;Second line\n    &nbsp;&nbsp;Third line\n    \"\"\"\n    iex>Bunch.Markdown.hard_indent(text, 1)\n    \"\"\"\n    &nbsp;First line\n    &nbsp;Second line\n    &nbsp;Third line\n    \"\"\"","ref":"Bunch.Markdown.html#hard_indent/2-examples"},{"type":"function","title":"Bunch.Markdown.indent/2","doc":"Indents whole block of text by specified number of spaces","ref":"Bunch.Markdown.html#indent/2"},{"type":"function","title":"Examples - Bunch.Markdown.indent/2","doc":"iex>Bunch.Markdown.indent(\"text\")\n    \"  text\"\n\n    iex>text = \"\"\"\n    ...>First line\n    ...>Second line\n    ...>Third line\n    ...>\"\"\"\n    iex>Bunch.Markdown.indent(text)\n    \"\"\"\n      First line\n      Second line\n      Third line\n    \"\"\"\n    iex>Bunch.Markdown.indent(text, 4)\n    \"\"\"\n        First line\n        Second line\n        Third line\n    \"\"\"","ref":"Bunch.Markdown.html#indent/2-examples"},{"type":"module","title":"Bunch.Math","doc":"A bunch of math helper functions.","ref":"Bunch.Math.html"},{"type":"function","title":"Bunch.Math.div_rem/2","doc":"Applies `div/2` and `rem/2` to arguments and returns results as a tuple.","ref":"Bunch.Math.html#div_rem/2"},{"type":"function","title":"Example - Bunch.Math.div_rem/2","doc":"iex> Bunch.Math.div_rem(10, 4)\n    {div(10, 4), rem(10, 4)}","ref":"Bunch.Math.html#div_rem/2-example"},{"type":"function","title":"Bunch.Math.div_rem/3","doc":"Works like `div_rem/2` but allows to accumulate remainder.\n\nUseful when an accumulation of division error is not acceptable, for example\nwhen you need to produce chunks of data every second but need to make sure there\nare 9 chunks per 4 seconds on average. You can calculate `div_rem(9, 4)`,\nkeep the remainder, pass it to subsequent calls and every fourth result will be\nbigger than others.","ref":"Bunch.Math.html#div_rem/3"},{"type":"function","title":"Example - Bunch.Math.div_rem/3","doc":"iex> 1..10 |> Enum.map_reduce(0, fn _, err ->\n    ...>  Bunch.Math.div_rem(9, 4, err)\n    ...>  end)\n    {[2, 2, 2, 3, 2, 2, 2, 3, 2, 2], 2}","ref":"Bunch.Math.html#div_rem/3-example"},{"type":"function","title":"Bunch.Math.max_multiple_lte/2","doc":"Returns the biggest multiple of `value` that is lower than or equal to `threshold`.","ref":"Bunch.Math.html#max_multiple_lte/2"},{"type":"function","title":"Examples - Bunch.Math.max_multiple_lte/2","doc":"iex> Bunch.Math.max_multiple_lte(4, 10)\n    8\n    iex> Bunch.Math.max_multiple_lte(2, 6)\n    6","ref":"Bunch.Math.html#max_multiple_lte/2-examples"},{"type":"function","title":"Bunch.Math.min_multiple_gte/2","doc":"Returns the smallest multiple of `value` that is greater than or equal to `threshold`.","ref":"Bunch.Math.html#min_multiple_gte/2"},{"type":"function","title":"Examples - Bunch.Math.min_multiple_gte/2","doc":"iex> Bunch.Math.min_multiple_gte(4, 10)\n    12\n    iex> Bunch.Math.min_multiple_gte(2, 6)\n    6","ref":"Bunch.Math.html#min_multiple_gte/2-examples"},{"type":"module","title":"Bunch.Module","doc":"A bunch of functions for easier manipulation on modules.","ref":"Bunch.Module.html"},{"type":"function","title":"Bunch.Module.apply/4","doc":"Works like `Kernel.apply/3` if `module` exports `fun_name/length(args)`,\notherwise returns `default`.\n\nDetermines if function is exported using `loaded_and_function_exported?/3`.","ref":"Bunch.Module.html#apply/4"},{"type":"function","title":"Bunch.Module.check_behaviour/2","doc":"Determines whether module implements a behaviour by checking a test function.\n\nChecked behaviour needs to define a callback with unique name and no arguments,\nthat should return `true`. This functions ensures that the module is loaded and\nchecks if it exports implementation of the callback that returns `true`. If\nall these conditions are met, `true` is returned. Otherwise returns `false`.","ref":"Bunch.Module.html#check_behaviour/2"},{"type":"function","title":"Bunch.Module.loaded_and_function_exported?/3","doc":"Ensures that module is loaded and checks whether it exports given function.","ref":"Bunch.Module.html#loaded_and_function_exported?/3"},{"type":"function","title":"Bunch.Module.struct/1","doc":"Returns instance of struct defined in given module, if the module defines struct.\nOtherwise returns `nil`.\n\nRaises if struct has any required fields.","ref":"Bunch.Module.html#struct/1"},{"type":"module","title":"Bunch.Retry","doc":"A bunch of helpers for handling scenarios when some actions should be repeated\nuntil it succeeds.","ref":"Bunch.Retry.html"},{"type":"function","title":"Bunch.Retry.retry/3","doc":"Calls `fun` function until `arbiter` function decides to stop.\n\nPossible options are:\n- times - limits amount of retries (first evaluation is not considered a retry)\n- duration - limits total time of execution of this function, but breaks only\nbefore subsequent retry\n- delay - introduces delay (`:timer.sleep/1`) before each retry","ref":"Bunch.Retry.html#retry/3"},{"type":"function","title":"Examples - Bunch.Retry.retry/3","doc":"iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Retry.retry(fn -> Agent.get_and_update(pid, &{&1, &1+1}) end, & &1 > 3)\n    4\n    iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Retry.retry(fn -> Agent.get_and_update(pid, &{&1, &1+1}) end, & &1 > 3, times: 10)\n    4\n    iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Retry.retry(fn -> Agent.get_and_update(pid, &{&1, &1+1}) end, & &1 > 3, times: 2)\n    2\n    iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Retry.retry(\n    ...> fn -> :timer.sleep(100); Agent.get_and_update(pid, &{&1, &1+1}) end,\n    ...> & &1 > 3,\n    ...> duration: 150\n    ...> )\n    1\n    iex> {:ok, pid} = Agent.start_link(fn -> 0 end)\n    iex> Bunch.Retry.retry(\n    ...> fn -> :timer.sleep(30); Agent.get_and_update(pid, &{&1, &1+1}) end,\n    ...> & &1 > 3,\n    ...> duration: 80, delay: 20\n    ...> )\n    1","ref":"Bunch.Retry.html#retry/3-examples"},{"type":"type","title":"Bunch.Retry.retry_option_t/0","doc":"Possible options for `retry/3`.","ref":"Bunch.Retry.html#t:retry_option_t/0"},{"type":"module","title":"Bunch.ShortRef","doc":"A wrapper over Erlang/Elixir references that makes them more readable and visually\ndistinguishable.","ref":"Bunch.ShortRef.html"},{"type":"module","title":"Erlang references - Bunch.ShortRef","doc":"When printed, Erlang references are quite long: `#Reference `.\nMoreover, since they're based on an incremented counter, it's hard to distinguish\nbetween a two created within the same period of time, like `#Reference `\nand `#Reference `.","ref":"Bunch.ShortRef.html#module-erlang-references"},{"type":"module","title":"Bunch.ShortRef - Bunch.ShortRef","doc":"`t:Bunch.ShortRef.t/0` stores a usual reference along with first 4 bytes\nof its SHA1 hash and when inspected prints only 8 hex digits prepended with `#`.\nThanks to use of the hash function, similar references have totally different\nstring representations - the case from the previous example would be `#60e0fd2d`\nand `#d4208051`.","ref":"Bunch.ShortRef.html#module-bunch-shortref"},{"type":"module","title":"When to use - Bunch.ShortRef","doc":"`Bunch.ShortRef` should be used when a reference is to be printed or logged.\nIt should NOT be used when creating lots of references as it adds a significant\nperformance overhead.","ref":"Bunch.ShortRef.html#module-when-to-use"},{"type":"function","title":"Bunch.ShortRef.new/1","doc":"Creates a short reference.\n\n    iex> IEx.Helpers.ref(0, 1, 2, 3) |> Bunch.ShortRef.new() |> inspect()\n    \"#82c033ef\"\n    iex>  > = Bunch.ShortRef.new() |> inspect()\n    iex> Base.decode16(hash, case: :lower) |> elem(0)\n    :ok","ref":"Bunch.ShortRef.html#new/1"},{"type":"type","title":"Bunch.ShortRef.t/0","doc":"","ref":"Bunch.ShortRef.html#t:t/0"},{"type":"module","title":"Bunch.Struct","doc":"A bunch of functions for easier manipulation on structs.","ref":"Bunch.Struct.html"},{"type":"function","title":"Bunch.Struct.delete_in/2","doc":"Wraps `Bunch.Access.delete_in/2` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#delete_in/2"},{"type":"function","title":"Bunch.Struct.get_and_update_in/3","doc":"Wraps `Bunch.Access.get_and_update_in/3` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#get_and_update_in/3"},{"type":"function","title":"Bunch.Struct.get_in/2","doc":"Wraps `Bunch.Access.get_in/2` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#get_in/2"},{"type":"function","title":"Bunch.Struct.pop_in/2","doc":"Wraps `Bunch.Access.pop_in/2` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#pop_in/2"},{"type":"function","title":"Bunch.Struct.put_in/3","doc":"Wraps `Bunch.Access.put_in/3` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#put_in/3"},{"type":"function","title":"Bunch.Struct.update_in/3","doc":"Wraps `Bunch.Access.update_in/3` to make it work with structs that do not\nimplement `Access` behaviour.","ref":"Bunch.Struct.html#update_in/3"},{"type":"module","title":"Bunch.Type","doc":"A bunch of commonly used types.","ref":"Bunch.Type.html"},{"type":"type","title":"Bunch.Type.stateful_t/2","doc":"Represents a value along with state.","ref":"Bunch.Type.html#t:stateful_t/2"},{"type":"type","title":"Bunch.Type.stateful_try_t/1","doc":"Represents a `t:try_t/0` value along with state.","ref":"Bunch.Type.html#t:stateful_try_t/1"},{"type":"type","title":"Bunch.Type.stateful_try_t/2","doc":"Represents a `t:try_t/1` value along with state.","ref":"Bunch.Type.html#t:stateful_try_t/2"},{"type":"type","title":"Bunch.Type.try_t/0","doc":"Represents result of an operation that may succeed or fail.","ref":"Bunch.Type.html#t:try_t/0"},{"type":"type","title":"Bunch.Type.try_t/1","doc":"Represents result of an operation that may return something or fail.","ref":"Bunch.Type.html#t:try_t/1"},{"type":"module","title":"Bunch.Typespec","doc":"A bunch of typespec-related helpers.","ref":"Bunch.Typespec.html"},{"type":"macro","title":"Bunch.Typespec.@/1","doc":"**This macro is deprecated. Use `Bunch.Typespec.enum_to_alternative/1` instead.**\n\nAllows to define a type in form of `t :: x | y | z | ...` and a module parameter\nin form of `@t [x, y, z, ...]` at once.","ref":"Bunch.Typespec.html#@/1"},{"type":"macro","title":"Example - Bunch.Typespec.@/1","doc":"defmodule Abc do\n      use Bunch.Typespec\n      @list_type t :: [:a, :b, :c]\n      @spec get_at(0..2) :: t\n      def get_at(x), do: @t |> Enum.at(x)\n    end\n\n    Abc.get_at(1) # -> :b","ref":"Bunch.Typespec.html#@/1-example"},{"type":"function","title":"Bunch.Typespec.enum_to_alternative/1","doc":"Converts an enumerable of terms to AST of alternative type of these terms.\n\nUseful for defining a type out of a list of constants.","ref":"Bunch.Typespec.html#enum_to_alternative/1"},{"type":"function","title":"Examples - Bunch.Typespec.enum_to_alternative/1","doc":"iex> defmodule Example do\n    ...>   @values [1, :a, {true, 3 * 4}]\n    ...>   @type value :: unquote(Bunch.Typespec.enum_to_alternative(@values))\n    ...>   @spec get_value(0..2) :: value\n    ...>   def get_value(i), do: Enum.at(@values, i)\n    ...> end\n    iex> Example.get_value(1)\n    :a\n\n    iex> Bunch.Typespec.enum_to_alternative([1, :a, {true, 3 * 4}])\n    quote do\n      1 | :a | {true, 12}\n    end","ref":"Bunch.Typespec.html#enum_to_alternative/1-examples"},{"type":"extras","title":"Bunch","doc":"# Bunch\n\n[![Hex.pm](https://img.shields.io/hexpm/v/bunch.svg)](https://hex.pm/packages/bunch)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/bunch/)\n[![CircleCI](https://circleci.com/gh/membraneframework/bunch.svg?style=svg)](https://circleci.com/gh/membraneframework/bunch)\n\nBunch is a bunch of helper functions, intended to make life easier.\n\nAPI documentation is available at [HexDocs](https://hexdocs.pm/bunch/)","ref":"readme.html"},{"type":"extras","title":"Installation - Bunch","doc":"Add the following line to your `deps` in `mix.exs`.  Run `mix deps.get`.\n\n```elixir\n{:bunch, \"~> 1.6\"}\n```","ref":"readme.html#installation"},{"type":"extras","title":"Copyright and License - Bunch","doc":"Copyright 2018, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2018 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}